﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using OpenHardware.Properties;
using System.Diagnostics;

namespace OpenHardware
{
    public partial class frmMain : Form
    {
        SerialPort serial_connection = new SerialPort(Settings.Default.com_port, Settings.Default.baud_rate);
        byte[] ServoMoves = { 1, 2, 3 }; 

        public frmMain()
        {
            InitializeComponent();
        }

        private void LoadCOMPortList()
        {
            cmbComPort.Items.Clear();   // Clear the list of the COM ports
            foreach (string port in SerialPort.GetPortNames()) // Loop through all the available COM ports
            {
                cmbComPort.Items.Add(port); // Add the port to the list
            }
            
            // Check if the default COM port is in the list and if so select it
            cmbComPort.SelectedIndex = cmbComPort.FindString(Settings.Default.com_port);
            
            if (cmbComPort.SelectedIndex < 0) // If none of the ports is selected select the first one
            {
                cmbComPort.SelectedIndex = 0;
                Settings.Default.com_port = cmbComPort.Text; // Save the new selection
                Settings.Default.Save();
            }
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            LoadCOMPortList();  // Load all the COM ports
            // Load the last selected COM port
            cmbBRate.SelectedIndex = cmbBRate.FindString(Settings.Default.baud_rate.ToString());
            // Attach a DataReceived event to get callback from the development board
            serial_connection.DataReceived += new SerialDataReceivedEventHandler(serial_connection_DataReceived);
        }

        void serial_connection_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            // Nothing here
        }

        private void btnComReload_Click(object sender, EventArgs e)
        {
            // Reload COM port data to the list and select the last used port
            LoadCOMPortList(); 
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            try
            {
                // Use the button text as an indecator for conneciton
                if (btnConnect.Text == "Connect")
                {
                    // Load the information to the serial port for connection
                    SetupSerialPort(ref serial_connection, Settings.Default.com_port, Settings.Default.baud_rate, 8);
                    serial_connection.Open(); // Create a connection
                    btnConnect.Text = "Disconnect"; // Change the button caption
                }
                else
                {
                    serial_connection.Close(); // Close the connection
                    btnConnect.Text = "Connect"; // Change the button caption
                }
            }
            catch
            {
                btnConnect.Text = "Connect"; // Faild to connect
            }
        }
        
        private void SetupSerialPort(ref SerialPort sPort, string ComPort, int BaudRate, int DataBits)
        {
            sPort.PortName = ComPort;
            sPort.BaudRate = BaudRate;
            sPort.DataBits = 8;
            sPort.DtrEnable = true;
        }

        private void cmbBRate_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Save the information on change
            Settings.Default.baud_rate = Convert.ToInt32(cmbBRate.Text);
            Settings.Default.Save();
        }

        private void cmbComPort_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Save the information on change
            Settings.Default.com_port = cmbComPort.Text;
            Settings.Default.Save();
        }


        private void frmMain_KeyDown(object sender, KeyEventArgs e)
        {
            byte[] buffer_2_send = new byte[2];
            buffer_2_send[0] = Convert.ToByte(tbSpeed.Value);
            e.Handled = true;
            if (serial_connection.IsOpen)
            {
                switch (e.KeyCode)
                {
                    case Keys.Left:
                        buffer_2_send[1] = ServoMoves[0];
                        serial_connection.Write(buffer_2_send, 0, 2);
                        lblLeft.ForeColor = Color.Green;
                        break;
                    case Keys.Right:
                        buffer_2_send[1] = ServoMoves[1];
                        serial_connection.Write(buffer_2_send, 0, 2);
                        lblRight.ForeColor = Color.Green;
                        break;
                    case Keys.Home:
                        buffer_2_send[1] = ServoMoves[2];
                        serial_connection.Write(buffer_2_send, 0, 2);
                        lblHome.ForeColor = Color.Green;
                        break;
                    case Keys.PageUp:
                        if (tbSpeed.Value < tbSpeed.Maximum) tbSpeed.Value++;
                        break;
                    case Keys.PageDown:
                        if (tbSpeed.Value > tbSpeed.Minimum) tbSpeed.Value--;
                        break;
                }
            }
        }

        private void frmMain_KeyUp(object sender, KeyEventArgs e)
        {
            lblHome.ForeColor = Color.Black;
            lblRight.ForeColor = Color.Black;
            lblLeft.ForeColor = Color.Black;
        }
    }
}
